package com.ruoyi.system.service.impl;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.exception.job.TaskException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.quartz.domain.SysJob;
import com.ruoyi.quartz.service.ISysJobService;
import com.ruoyi.system.domain.Apply;
import com.ruoyi.system.domain.WeverseAgree;
import com.ruoyi.system.mapper.ApplyMapper;
import org.quartz.SchedulerException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.PerformMapper;
import com.ruoyi.system.domain.Perform;
import com.ruoyi.system.service.IPerformService;
import org.springframework.transaction.annotation.Transactional;

/**
 * 演出Service业务层处理
 *
 * @author ruoyi
 * @date 2025-01-26
 */
@Service
public class PerformServiceImpl implements IPerformService {
    @Autowired
    private PerformMapper performMapper;
    @Autowired
    private ISysJobService jobService;
    @Autowired
    private ApplyMapper applyMapper;

    /**
     * 查询演出
     *
     * @param id 演出主键
     * @return 演出
     */
    @Override
    public Perform selectPerformById(Long id) {
        return performMapper.selectPerformById(id);
    }

    @Override
    public Perform getInfoNormal(Long id) {
        return performMapper.getInfoNormal(id);
    }

    /**
     * 查询演出列表
     *
     * @param perform 演出
     * @return 演出
     */
    @Override
    public List<Perform> selectPerformList(Perform perform) {
        return performMapper.selectPerformList(perform);
    }

    /**
     * 查询演出列表
     *
     * @return 演出
     */
    @Override
    public List<Perform> getAll() {
        return performMapper.selectAll();
    }

    /**
     * 新增演出
     *
     * @param perform 演出
     * @return 结果
     */
    @Override
    @Transactional
    public int insertPerform(Perform perform, String userName) throws Exception {
        if (perform.getStartTime().compareTo(new Date()) <= 0) {
            throw new Exception("开始时间必须大于当前时间");
        }
        perform.setCreateTime(DateUtils.getNowDate());
        //未开始
        perform.setStatus("1");

        int result = performMapper.insertPerform(perform);
        initRobJob(perform.getId(), perform.getName(), perform.getStartTime(), userName);
        initGetJob(perform.getId(), perform.getName(), perform.getStartTime(), perform.getAdvTime(), userName);
        return result;
    }

    /**
     * 抢票任务
     */
    private void initRobJob(Long pid, String name, Date date, String userName) throws SchedulerException, TaskException {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.HOUR_OF_DAY, -1);
        calendar.add(Calendar.MINUTE, -2);
        // 获取年、月、日、时、分、秒
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH) + 1; // 月份从0开始，所以加1
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        int hour = calendar.get(Calendar.HOUR_OF_DAY); // 24小时制
        int minute = calendar.get(Calendar.MINUTE);
        int second = calendar.get(Calendar.SECOND);
        String cron = String.format("%d %d %d %d %d ? %d", second, minute, hour, day, month, year);

        // 定义日期格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        // 格式化当前日期和时间
        String formattedDate = sdf.format(date);

        SysJob job = new SysJob();
        job.setJobName(name + " " + formattedDate);
        job.setConcurrent("0");
        job.setCronExpression(cron);
        job.setInvokeTarget("ryTask.rob(" + pid + "L)");
        job.setJobGroup("DEFAULT");
        job.setMisfirePolicy("3");
        job.setStatus("0");
        job.setCreateBy(userName);
        jobService.insertJob(job);

        job.setInvokeTarget("ryTask.rob(" + pid + "L, " + job.getJobId() + "L)");
        jobService.updateJob(job);
    }

    /**
     * 获取参数任务
     */
    private void initGetJob(Long pid, String name, Date date, Long advTime, String userName) throws SchedulerException, TaskException {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.HOUR_OF_DAY, -1);
        calendar.add(Calendar.MINUTE, -advTime.intValue());
        // 获取年、月、日、时、分、秒
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH) + 1; // 月份从0开始，所以加1
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        int hour = calendar.get(Calendar.HOUR_OF_DAY); // 24小时制
        int minute = calendar.get(Calendar.MINUTE);
        int second = calendar.get(Calendar.SECOND);
        String cron = String.format("%d %d %d %d %d ? %d", second, minute, hour, day, month, year);

        // 定义日期格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        // 格式化当前日期和时间
        String formattedDate = sdf.format(date);

        SysJob job = new SysJob();
        job.setJobName("获取参数 " + name + " " + formattedDate);
        job.setConcurrent("0");
        job.setCronExpression(cron);
        job.setInvokeTarget("ryTask.get(" + pid + "L)");
        job.setJobGroup("DEFAULT");
        job.setMisfirePolicy("3");
        job.setStatus("0");
        job.setCreateBy(userName);
        jobService.insertJob(job);

        job.setInvokeTarget("ryTask.get(" + pid + "L, " + job.getJobId() + "L)");
        jobService.updateJob(job);
    }

    /**
     * 修改演出
     *
     * @param perform 演出
     * @return 结果
     */
    @Override
    public int updatePerform(Perform perform) throws Exception {
        if (perform.getStartTime().compareTo(new Date()) <= 0) {
            throw new Exception("开始时间必须大于当前时间");
        }
        perform.setUpdateTime(DateUtils.getNowDate());
        return performMapper.updatePerform(perform);
    }

    /**
     * 批量删除演出
     *
     * @param ids 需要删除的演出主键
     * @return 结果
     */
    @Override
    public int deletePerformByIds(Long[] ids) {
        return performMapper.deletePerformByIds(ids);
    }

    /**
     * 删除演出信息
     *
     * @param id 演出主键
     * @return 结果
     */
    @Override
    public int deletePerformById(Long id) {
        return performMapper.deletePerformById(id);
    }

    @Override
    public boolean refresh(Long id) throws Exception {
        Perform perform = performMapper.selectPerformById(id);
        if(perform==null){
            throw  new Exception("演出不存在");
        }
        //人员
        Apply apply = new Apply();
        apply.setPerformId(id);
        List<Apply> applies = applyMapper.selectApplyListRob(apply);
        if (applies.isEmpty()) {
           throw  new Exception("人员不存在");
        }
        for (Apply a : applies) {
            // 创建HttpClient实例
            HttpClient httpClient = HttpClient.newBuilder()
                    .connectTimeout(Duration.ofSeconds(10))
                    .build();

            HttpRequest request = HttpRequest.newBuilder()
                    .uri(new URI("https://faneventapi.weverse.io/api/v1/user/events/"+perform.getNum()))
                    .GET()
                    .header("Authorization", "Bearer " + a.getToken())
                    .header("Origin", "https://weverse.io")
                    .header("Referer", "https://weverse.io/")
                    .header("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36")
                    .build();
            // 发送请求并获取响应
            HttpResponse<String> response = httpClient.send(request, HttpResponse.BodyHandlers.ofString());
            if (response.statusCode() == 200) {
                WeverseAgree weverseAgree = JSONObject.parseObject(response.body(), WeverseAgree.class);
                System.out.println(weverseAgree);
                if (weverseAgree == null || weverseAgree.getUserStatus() == null) {
                    throw new Exception("状态未找到");
                }

                if ("DROPPED".equals(weverseAgree.getUserStatus())) {
                    Apply apply1 = new Apply();
                    apply1.setId(a.getId());
                    apply1.setStatus("4");
                    applyMapper.updateApply(apply1);
                }else if ("testaaa".equals(weverseAgree.getUserStatus())){
                    Apply apply1 = new Apply();
                    apply1.setId(a.getId());
                    apply1.setStatus("3");
                    applyMapper.updateApply(apply1);
                }

            }
        }
        return true;
    }
}
